
/*
**	FILE:			HostProbe.java
**
**	CREATED:		12th April 2000
**
**	DESCRIPTION:	Manages ScanPort threads to scan ports of a given host.
**					HostProbe spawns one ScanPort thread per port, while 
**					limiting the number of concurrent active threads.
**
**	CHANGELOG:
**	DATE		TIME	DESCRIPTION
**	-----------	-----	------------------------------------------------------
**						
*/

import java.io.*;
import java.net.*;

class	HostProbe extends Thread
{
	InetAddress		host;
	int				startPort;
	int				endPort;
	int				maxThreads;
	static int			oldPort, count;
	ThreadGroup		portscans;
	PrintStream		ostream;
	int				numPorts;
	Semaphore		sem;

	HostProbe (InetAddress host, int startPort, int endPort, int maxThreads,
				PrintStream ostream)
	{
		/*
		**	Save parameters
		*/
		this.host = host;
		this.startPort = startPort;
		this.endPort = endPort;
		this.maxThreads = maxThreads;
		this.ostream = ostream;
		oldPort = 0;count =1;

		/*
		**	This semaphore will be used by PortProbe threads to signal when
		**	they complete.
		*/
		sem = new Semaphore ();

;
	}
	
	public void run ()
	{
		int		portsDone = 0;

		ostream.println ("------------------------------------------------------------------------------");
		ostream.println ("PORT  MESSAGE");

		/*
		**	Start process timer
		*/
		ElapsedTimer timer = new ElapsedTimer ();
		timer.Start ();

		portscans = new ThreadGroup ("portscans");

		int port = startPort;

		while (port <= endPort)
		{
			/*
			**	If number of active PortProbe threads is less than
			**	maximum allowed, kick off another one.
			*/
			if (portscans.activeCount() < maxThreads)
			{
				/* 
				**	Calculate percentage complete 
				*/ 
				if (oldPort == count - 100)
				{
					System.out.print (port+"\r");
					oldPort=count;
				}
				count++;
				
				 
				/* 
				**	Wait for the next PortProbe thread to end 
				*/ 
				(new PortProbe (portscans, host, port, ostream, sem)).start();
				port++;

				
			}
			else
			{
				
				/* 
				**	Calculate percentage complete 
				*/ 
				if (oldPort == count - 100)
				{
					System.out.print (port+"\r");
					oldPort=count;
				}
				count++;
				
				
				/* 
				**	Wait for the next PortProbe thread to end 
				*/ 
				
				sem.get();
			}
		}

		/*
		**	Wait for all active PortProbe threads to finish
		*/
		while (portscans.activeCount() > 0)
		{
			sem.get();
		}

		ostream.println ("------------------------------------------------------------------------------");
		ostream.println ("Probed ports "+startPort+" to "+endPort+" in "+
						(timer.Stop()/1000)+" seconds.");
	}
}

