
/*
**	FILE:			PortProbe.java
**
**	CREATED:		12th April 2000
**
**	DESCRIPTION:	Tries to connect to a given host+port.
**					If the port is open, it attempts to write to the port
**					and read any response. The port number and response (if
**					any) is written to a supplied PrintStream.
**
**	CHANGELOG:
**	DATE		TIME	DESCRIPTION
**	-----------	-----	------------------------------------------------------
**
*/

import java.io.*;
import java.net.*;

class	PortProbe extends Thread
{
	InetAddress	host; 
	int			port; 
	PrintStream	ostream;
	Semaphore	sem;
	Socket		socket;

	PortProbe (ThreadGroup group, InetAddress host, int port, PrintStream ostream, Semaphore sem)
	{
		super (group, ""+port);
		this.host = host;
		this.port = port;
		this.ostream = ostream;
		this.sem = sem;
	}

	private void	endProbe ()
	{
		/*
		**	Close the socket
		*/
		if (socket != null)
		{
			try
			{
				socket.close ();
			}
			catch (IOException e)
			{
			}
		}

		/*
		**	Signal that this probe is complete
		*/
		sem.put();
	}


	public void run ()
	{
		byte b[] = " ".getBytes();

		/*
		**	Connect to remote host port and send data
		*/
		try
		{
			socket = new Socket (host, port);
			socket.setSoTimeout (4000);
			socket.getOutputStream().write(b);
		}
		catch (IOException e)
		{
			endProbe ();
			return;
		}

		/*
		**	Attempt to read any response
		*/
		String s = null;
		try
		{
			s = (new BufferedReader(new InputStreamReader(socket.getInputStream ()))).readLine();
 		}
		catch (IOException e)
		{
		}

		/*
		**	Write result to output stream
		*/
		String tempString=""+port+" ";
		if (s != null)
		{
			tempString = tempString + "   " + s;
		}
		ostream.println (tempString);

		endProbe ();
	} 
}

