
/*
**	FILE:			Scanner.java
**
**	CREATED:		23 March 2000 (i think)
**	
**	VERSION:		2
**
**	DESCRIPTION:	Scans a port range of a target host, logs open ports
**			and sign-on message to a log file.
**
**	CHANGELOG:
**	DATE		TIME		DESCRIPTION
**	-----------	-----		------------------------------------------------------
**
**	12-Apr-2000 16:02		Do lookup of target address in main, and pass
**					internet address to scan thread.
**					Log file parameter removed. Output is written
**					to file 'host'.LOG
**
**	12-Apr-2000 19:00		Moved classes to separate files.
**
**	17-Apr-2000 17:00 - 19:30 	Changed output file back to args[3] due to the 
**					scenario where the user has read permission but
**					not write permission to the current directory.
**
**					Removed class TeeOutputStream & ProgressReport.
**					I changed the output back to just a log file so
**					the progress status could be seen better. I removed
**					ProgressReport because there where problems with the
**					frequency in wich it was updating (it seemed to me 
**					that it would only update when scanning an open port)
**					The problem has now been fixed by integrating the 
**					method into HostProbe.
**						
*/

import java.io.*;
import java.net.*;

public class Scanner extends Thread
{
	static final String	VERSION = "v2";
	static final int	DEF_THREADS = 100;

	static InetAddress	hostAddr;
	static String		toFile;
	static int			startPort, endPort;
	static PrintStream	outFile;
	static int			numThreads;
	static int			numPorts;
	static String		logFile;

	/*
	**	Main entry point for application
	*/
	public static void main (String args[])
	{
		try
		{
			/*
			**	Display banner
			*/
			displayWelcome ();

			/*
			**	Parse command line parameters
			*/
			parseCommandLine (args);

			/*
			**	Create and start the scanning thread
			*/
			System.out.println();
			System.out.println("Scanned ports...");
			(new HostProbe(hostAddr, startPort, endPort, numThreads, outFile)).start();

		}
		catch (Exception e)
		{
			System.out.println ();
			System.out.println ("ERROR: "+e.getMessage());
		}
	}


	static private void	displayWelcome ()
	{
		/*
		**	Display version and welcome message
		*/
		System.out.println();
		System.out.println ("           Jport a Multi-threaded Java port scanner "+VERSION);
		System.out.println ();
		System.out.println ("Written by snafu & modified by Dave (revhead@dalsoft.bpc-users.org)");
		System.out.println ();
		System.out.println ();
		System.out.println (" *****************************************************************");
		System.out.println ("**Originaly distributed by overloaded.org (http://overloaded.org)**");
		System.out.println (" *****************************************************************");
		System.out.println ();
		System.out.println ("          -]] Report any bugs to jport@overloaded.org [[-");
	}

	static private void	parseCommandLine (String args[])
						throws Exception
	{
		/*
		**	Parse essential command line.
		**	If syntax is wrong then display the correct syntax and exit.
		*/
		String hostTxt=null;
		try
		{
			hostTxt=args[0];
			logFile=args[3];
		}
		catch (Exception e)
		{
			displayUsage ();
			throw new Exception ("Incorrect command syntax");
		}

		/*
		**	Parse optional parameter(s)
		*/
		startPort=1;
		endPort=65535;
		numThreads = DEF_THREADS;
		try
		{
			startPort=Integer.parseInt(args[1]); 
			endPort=Integer.parseInt(args[2]); 
			numThreads=Integer.parseInt(args[4]); 
		}
		catch (ArrayIndexOutOfBoundsException e)
		{
		}
		catch (NumberFormatException e)
		{
			displayUsage ();
			throw new Exception ("Invalid parameter format");
		}

		/*
		**	Lookup address of target host
		*/
		try
		{
			hostAddr = InetAddress.getByName (hostTxt);
		}
		catch (UnknownHostException e)
		{
			throw new Exception ("Unknown host "+hostTxt);
		}

		/*
		**	Create the output file, and write header.
		**	If can't create output file, display an error message and exit.
		*/
		try
		{
			outFile = new PrintStream (new FileOutputStream(logFile, false));
			outFile.println ("Scan for "+args[0]+" starting port "+startPort+
								" ending on port "+endPort);
		}
		catch (IOException fileIOErr)
		{
 			throw new Exception ("Cannot write to file "+logFile+". Do you have permission?");
		}
	}


	static private void	displayUsage ()
	{
		System.out.println ();
		System.out.println ("USAGE: java Scanner [host] [start] [end] [log file] [threads]");
		System.out.println ("       host = name or address of host to scan");
		System.out.println ("       start = first port to scan (default = 1)");
		System.out.println ("       end = last port to scan (default = 65535)");
		System.out.println ("       log file = the file to log to");
		System.out.println ("       threads = number of threads to use (default = "+DEF_THREADS+")");
		System.out.println ("   Output is written to console, and to file 'host'.LOG");
	}
}




